@@ -5,10 +5,10 @@
 #include "socket.h"
 
 class CControlSocket;
-class CProxySocket final : protected CEventHandler, public CBackend
+class CProxySocket final : protected fz::event_handler, public CBackend
 {
 public:
-	CProxySocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner);
+	CProxySocket(event_handler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner);
 	virtual ~CProxySocket();
 
 	enum ProxyState {
@@ -25,6 +25,7 @@ public:
 
 		proxytype_count
 	};
+	static wxString Name(ProxyType t);
 
 	int Handshake(ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass);
 
@@ -65,9 +66,9 @@ protected:
 	int m_recvBufferPos{};
 	int m_recvBufferLen{};
 
-	virtual void operator()(CEventBase const& ev);
+	virtual void operator()(fz::event_base const& ev);
 	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
-	void OnHostAddress(CSocketEventSource* source, wxString const& address);
+	void OnHostAddress(CSocketEventSource* source, std::string const& address);
 
 	void OnReceive();
 	void OnSend();
