@@ -3,27 +3,25 @@
 
 #include "socket.h"
 
-// Strongly typed enum would be nice, but we need to support older compilers still.
-namespace dependency {
-enum type {
+enum class lib_dependency
+{
 	wxwidgets,
 	gnutls,
 	sqlite,
 	count
 };
-}
 
-wxString GetDependencyName( dependency::type d );
-wxString GetDependencyVersion( dependency::type d );
+wxString GetDependencyName(lib_dependency d);
+wxString GetDependencyVersion(lib_dependency d);
 
-wxString ListTlsCiphers(const wxString& priority);
+wxString ListTlsCiphers(wxString const& priority);
 
 // Microsoft, in its insane stupidity, has decided to make GetVersion(Ex) useless, starting with Windows 8.1,
 // this function no longer returns the operating system version but instead some arbitrary and random value depending
 // on the phase of the moon.
 // This function instead returns the actual Windows version. On non-Windows systems, it's equivalent to
 // wxGetOsVersion
-bool GetRealOsVersion( int& major, int& minor );
+bool GetRealOsVersion(int& major, int& minor);
 
 template<typename Derived, typename Base>
 std::unique_ptr<Derived>
