@@ -20,6 +20,7 @@
 #include "drop_target_ex.h"
 #include "edithandler.h"
 #include "filelist_statusbar.h"
+#include "graphics.h"
 #include "local_recursive_operation.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
@@ -269,6 +270,7 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 	m_state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
+	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	m_dropTarget = -1;
 
@@ -293,6 +295,8 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 	SetDropTarget(new CLocalListViewDropTarget(this));
 
 	EnablePrefixSearch(true);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
 }
 
 CLocalListView::~CLocalListView()
@@ -615,17 +619,19 @@ void CLocalListView::DisplayDrives()
 {
 	int count = m_fileData.size();
 
-	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
-	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+	std::list<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
+	for (auto it = drives.begin(); it != drives.end(); ++it) {
 		wxString drive = *it;
-		if (drive.Right(1) == _T("\\"))
+		if (drive.Right(1) == _T("\\")) {
 			drive.RemoveLast();
+		}
 
 		CLocalFileData data;
 		data.name = drive;
 		data.label = fz::sparse_optional<wxString>(data.name);
 		data.dir = true;
 		data.size = -1;
+		data.icon = GetIconIndex(iconType::dir, _T(""), false);
 
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(count);
@@ -1250,12 +1256,13 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 		if (i == (int)m_indexMapping.size())
 			break;
 	}
-	if (!focused.empty())
-	{
-		if (firstSelected != -1)
+	if (!focused.empty()) {
+		if (firstSelected != -1) {
 			SetItemState(firstSelected, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-		else
+		}
+		else {
 			SetItemState(0, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
+		}
 	}
 }
 
@@ -1265,6 +1272,9 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, con
 		DisplayDir(m_state.GetLocalDir());
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyCurrentFilter();
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
 		RefreshFile(data);
@@ -1338,7 +1348,7 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
+	fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
 	if (type == fz::local_filesys::unknown)
 		return;
 
@@ -1744,8 +1754,9 @@ int64_t CLocalListView::ItemGetSize(int index) const
 
 void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread)
+	if (!m_pVolumeEnumeratorThread) {
 		return;
+	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
@@ -1755,22 +1766,30 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	if (m_dir.GetPath() != _T("\\"))
+	if (m_dir.GetPath() != _T("\\")) {
 		return;
+	}
 
 	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
 		wxString drive = iter->volume;
 
 		unsigned int item, index;
-		for (item = 1; item < m_indexMapping.size(); ++item) {
+		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
-			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" "))
+			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" ")) {
 				break;
+			}
 		}
-		if (item >= m_indexMapping.size())
+		if (item >= m_indexMapping.size()) {
 			continue;
+		}
 
-		m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+		if (!iter->volumeName.empty()) {
+			m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+		}
+		if (iter->icon != -1) {
+			m_fileData[index].icon = iter->icon;
+		}
 
 		RefreshItem(item);
 	}
