@@ -217,10 +217,11 @@ protected:
 		}
 
 		CContextControl::_context_controls* controls = m_pMainFrame->m_pContextControl->GetControlsFromState(pState);
-		if (!controls)
+		if (!controls) {
 			return;
+		}
 
-		if (controls->tab_index == -1) {
+		if (!controls->used()) {
 			if (notification == STATECHANGE_REMOTE_IDLE || notification == STATECHANGE_SERVER)
 				pState->Disconnect();
 
@@ -236,12 +237,6 @@ protected:
 				}
 				else {
 					m_pMainFrame->SetTitle(pState->GetTitle() + _T(" - FileZilla"));
-					if (pServer->GetName().empty()) {
-						// Can only happen through quickconnect bar
-						CMenuBar* pMenuBar = dynamic_cast<CMenuBar*>(m_pMainFrame->GetMenuBar());
-						if (pMenuBar)
-							pMenuBar->ClearBookmarks();
-					}
 				}
 			}
 
@@ -429,7 +424,6 @@ CMainFrame::CMainFrame()
 
 	FixTabOrder();
 
-	RegisterOption(OPTION_LANGUAGE);
 	RegisterOption(OPTION_THEME);
 	RegisterOption(OPTION_THEME_ICONSIZE);
 	RegisterOption(OPTION_MESSAGELOG_POSITION);
@@ -772,42 +766,38 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		if (!pState) {
 			return;
 		}
-		CServer const* pServer = pState->GetServer();
+		
+		Site const old_site = pState->GetSite();
+		std::wstring sitePath = old_site.m_path;
 
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
-		if (!controls)
+		if (!controls) {
 			return;
-		if (!pServer && !controls->site_bookmarks->path.empty()) {
-			// Get server from site manager
-			std::unique_ptr<CSiteManagerItemData_Site> data = CSiteManager::GetSiteByPath(controls->site_bookmarks->path);
-			if (data) {
-				server = data->m_server;
-				pServer = &server;
-			}
-			else {
-				controls->site_bookmarks->path.clear();
-				controls->site_bookmarks->bookmarks.clear();
-				if (m_pMenuBar)
-					m_pMenuBar->UpdateBookmarkMenu();
-			}
 		}
 
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, controls->site_bookmarks->path, pServer);
+			CNewBookmarkDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, controls->site_bookmarks->path, pServer);
-
+			CBookmarksDialog dlg(this, sitePath, old_site.m_server ? &old_site.m_server : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
-			controls->site_bookmarks->bookmarks.clear();
-			CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-			if (m_pMenuBar)
-				m_pMenuBar->UpdateBookmarkMenu();
+			if (!sitePath.empty()) {
+				std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(sitePath, false);
+				if (site) {
+					for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+						CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+						if (!controls) {
+							continue;
+						}
+						controls->pState->UpdateSite(old_site.m_path, *site);
+					}
+				}
+			}
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_HELP_WELCOME")) {
@@ -861,7 +851,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteById(event.GetId());
+		std::unique_ptr<Site> pData = CSiteManager::GetSiteById(event.GetId());
 
 		if (!pData) {
 			event.Skip();
@@ -1241,9 +1231,10 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 
 	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 	if (controls) {
-		COptions::Get()->SetLastServer(controls->pState->GetLastServer());
+		Site site = controls->pState->GetLastSite();
+		COptions::Get()->SetLastServer(site.m_server);
 		COptions::Get()->SetOption(OPTION_LASTSERVERPATH, controls->pState->GetLastServerPath().GetSafePath());
-		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, controls->site_bookmarks ? controls->site_bookmarks->path : wxString());
+		COptions::Get()->SetOption(OPTION_LAST_CONNECTED_SITE, site.m_path);
 	}
 
 	for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
@@ -1267,21 +1258,17 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 void CMainFrame::OnReconnect(wxCommandEvent &)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
-
-	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle())
-		return;
-
-	CServer server = pState->GetLastServer();
-
-	if (server.GetLogonType() == ASK) {
-		if (!CLoginManager::Get().GetPassword(server, false))
-			return;
 	}
 
+	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		return;
+	}
+
+	Site site = pState->GetLastSite();
 	CServerPath path = pState->GetLastServerPath();
-	ConnectToServer(server, path, true);
+	ConnectToSite(site, path);
 }
 
 void CMainFrame::OnRefresh(wxCommandEvent &)
@@ -1313,82 +1300,75 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(const CServer* pServer /*=0*/)
+void CMainFrame::OpenSiteManager(CServer const* pServer)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
+	if (!pState) {
 		return;
+	}
 
 	CSiteManagerDialog dlg;
 
-	std::set<wxString> handled_paths;
+	std::set<std::wstring> handled_paths;
 	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
-	if (pServer)
-	{
+	if (pServer) {
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.server = *pServer;
 		connected_sites.push_back(connected_site);
 	}
 
-	for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
-	{
+	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls)
+		if (!controls) {
 			continue;
+		}
 
-		const wxString& path = controls->site_bookmarks->path;
-		if (path.empty())
+		Site const& site = controls->pState->GetSite();
+		std::wstring const& path = site.m_path;
+		if (path.empty()) {
 			continue;
-		if (handled_paths.find(path) != handled_paths.end())
+		}
+		if (handled_paths.find(path) != handled_paths.end()) {
 			continue;
+		}
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = controls->pState->GetLastServer();
+		connected_site.server = site.m_server;
 		connected_sites.push_back(connected_site);
 		handled_paths.insert(path);
 	}
 
-	if (!dlg.Create(this, &connected_sites, pServer))
+	if (!dlg.Create(this, &connected_sites, pServer)) {
 		return;
+	}
 
 	int res = dlg.ShowModal();
-	if (res == wxID_YES || res == wxID_OK)
-	{
+	if (res == wxID_YES || res == wxID_OK) {
 		// Update bookmark paths
-		for (size_t j = 0; j < connected_sites.size(); j++)
-		{
-			for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
-			{
-				CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-				if (!controls)
-					continue;
-
-				if (connected_sites[j].old_path != controls->site_bookmarks->path)
-					continue;
-
-				controls->site_bookmarks->path = connected_sites[j].new_path;
-
-				controls->site_bookmarks->bookmarks.clear();
-				CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-
-				break;
+		for (auto const& connected_site : connected_sites) {
+			std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(connected_site.new_path, false);
+			if (site) {
+				for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+					CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+					if (!controls) {
+						continue;
+					}
+					controls->pState->UpdateSite(connected_site.old_path, *site);
+				}
 			}
 		}
 	}
 
-	if (res == wxID_YES)
-	{
-		CSiteManagerItemData_Site data;
-		if (!dlg.GetServer(data))
+	if (res == wxID_YES) {
+		Site data;
+		if (!dlg.GetServer(data)) {
 			return;
+		}
 
 		ConnectToSite(data);
 	}
-
-	if (m_pMenuBar)
-		m_pMenuBar->UpdateBookmarkMenu();
 }
 
 void CMainFrame::OnSiteManager(wxCommandEvent& e)
@@ -1766,7 +1746,7 @@ void CMainFrame::UpdateLayout()
 	// Now the other panes
 	for (int i = 0; i < m_pContextControl->GetTabCount(); i++)
 	{
-		struct CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+		CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
 		if (!controls)
 			continue;
 
@@ -1891,40 +1871,80 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 	}
 }
 
-bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
+bool CMainFrame::ConnectToSite(Site & data, CServerPath const& path)
 {
+	// First check if we need to ask user for a password
 	if (data.m_server.GetLogonType() == ASK ||
 		(data.m_server.GetLogonType() == INTERACTIVE && data.m_server.GetUser().empty()))
 	{
-		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName()))
+		if (!CLoginManager::Get().GetPassword(data.m_server, false, data.m_server.GetName())) {
 			return false;
+		}
 	}
 
-	if (newTab)
-		m_pContextControl->CreateTab();
-
-	if (!ConnectToServer(data.m_server, data.m_remoteDir))
+	// Check if current state is already connected and if needed ask whether to open in new tab
+	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState) {
 		return false;
+	}
 
-	CState *pState = CContextManager::Get()->GetCurrentContext();
-	if (pState) {
-		if (!data.m_localDir.empty()) {
-			bool set = pState->SetLocalDir(data.m_localDir, 0, false);
-
-			if (set && data.m_sync) {
-				wxASSERT(!data.m_remoteDir.empty());
-				pState->SetSyncBrowse(true, data.m_remoteDir);
+	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		int action = COptions::Get()->GetOptionVal(OPTION_ALREADYCONNECTED_CHOICE);
+		if (action < 2) {
+			wxDialogEx dlg;
+			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED"))) {
+				return false;
 			}
+
+			if (action != 0) {
+				XRCCTRL(dlg, "ID_OLDTAB", wxRadioButton)->SetValue(true);
+			}
+			else {
+				XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->SetValue(true);
+			}
+
+			if (dlg.ShowModal() != wxID_OK) {
+				return false;
+			}
+
+			if (XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->GetValue()) {
+				action = 0;
+			}
+			else {
+				action = 1;
+			}
+
+			if (XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked()) {
+				action |= 2;
+			}
+			COptions::Get()->SetOption(OPTION_ALREADYCONNECTED_CHOICE, action);
 		}
 
-		if (data.m_comparison && pState->GetComparisonManager()) {
-			pState->GetComparisonManager()->CompareListings();
+		if (!(action & 1)) {
+			m_pContextControl->CreateTab();
+			pState = CContextManager::Get()->GetCurrentContext();
 		}
 	}
 
-	SetBookmarksFromPath(data.m_path);
-	if (m_pMenuBar)
-		m_pMenuBar->UpdateBookmarkMenu();
+	// Next tell the state to connect
+	if (!pState->Connect(data, path.empty() ? data.m_default_bookmark.m_remoteDir : path)) {
+		return false;
+	}
+
+	// Apply comparison and sync browsing options
+	// FIXME: Move to state?
+	if (!data.m_default_bookmark.m_localDir.empty()) {
+		bool set = pState->SetLocalDir(data.m_default_bookmark.m_localDir, 0, false);
+
+		if (set && data.m_default_bookmark.m_sync) {
+			wxASSERT(!data.m_default_bookmark.m_remoteDir.empty());
+			pState->SetSyncBrowse(true, data.m_default_bookmark.m_remoteDir);
+		}
+	}
+
+	if (data.m_default_bookmark.m_comparison && pState->GetComparisonManager()) {
+		pState->GetComparisonManager()->CompareListings();
+	}
 
 	return true;
 }
@@ -2298,7 +2318,7 @@ void CMainFrame::ProcessCommandLine()
 		pState->SetLocalDir(local);
 	}
 
-	wxString site;
+	std::wstring site;
 	if (pCommandLine->HasSwitch(CCommandLine::sitemanager)) {
 		if (COptions::Get()->GetOptionVal(OPTION_INTERFACE_SITEMANAGER_ON_STARTUP) == 0) {
 			Show();
@@ -2306,7 +2326,7 @@ void CMainFrame::ProcessCommandLine()
 		}
 	}
 	else if ((site = pCommandLine->GetOption(CCommandLine::site)) != _T("")) {
-		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteByPath(site);
+		std::unique_ptr<Site> pData = CSiteManager::GetSiteByPath(site);
 
 		if (pData) {
 			ConnectToSite(*pData);
@@ -2343,11 +2363,9 @@ void CMainFrame::ProcessCommandLine()
 				return;
 		}
 
-		CState* pState = CContextManager::Get()->GetCurrentContext();
-		if (!pState)
-			return;
-
-		ConnectToServer(server, path);
+		Site site;
+		site.m_server = server;
+		ConnectToSite(site, path);
 	}
 }
 
@@ -2543,84 +2561,6 @@ void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 	m_pContextControl->CloseTab(m_pContextControl->GetCurrentTab());
 }
 
-void CMainFrame::SetBookmarksFromPath(const wxString& path)
-{
-	if (!m_pContextControl)
-		return;
-
-	std::shared_ptr<CContextControl::_context_controls::_site_bookmarks> site_bookmarks;
-	for (int i = 0; i < m_pContextControl->GetTabCount(); i++) {
-		if (i == m_pContextControl->GetCurrentTab())
-			continue;
-
-		CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls || !controls->site_bookmarks || controls->site_bookmarks->path != path)
-			continue;
-
-		site_bookmarks = controls->site_bookmarks;
-		site_bookmarks->bookmarks.clear();
-	}
-	if (!site_bookmarks) {
-		site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
-		site_bookmarks->path = path;
-	}
-
-	CContextControl::_context_controls *controls = m_pContextControl->GetCurrentControls();
-	if (controls) {
-		controls->site_bookmarks = site_bookmarks;
-		CSiteManager::GetBookmarks(controls->site_bookmarks->path, controls->site_bookmarks->bookmarks);
-	}
-}
-
-bool CMainFrame::ConnectToServer(const CServer &server, const CServerPath &path /*=CServerPath()*/, bool isReconnect /*=true*/)
-{
-	CState* pState = CContextManager::Get()->GetCurrentContext();
-	if (!pState)
-		return false;
-
-	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
-		int action = COptions::Get()->GetOptionVal(OPTION_ALREADYCONNECTED_CHOICE);
-		if( action < 2 ) {
-			wxDialogEx dlg;
-			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED")))
-				return false;
-
-			if (action != 0)
-				XRCCTRL(dlg, "ID_OLDTAB", wxRadioButton)->SetValue(true);
-			else
-				XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->SetValue(true);
-
-			if (dlg.ShowModal() != wxID_OK)
-				return false;
-
-			if (XRCCTRL(dlg, "ID_NEWTAB", wxRadioButton)->GetValue()) {
-				action = 0;
-			}
-			else {
-				action = 1;
-			}
-
-			if( XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked() ) {
-				action |= 2;
-			}
-			COptions::Get()->SetOption(OPTION_ALREADYCONNECTED_CHOICE, action);
-		}
-
-		if( !(action & 1) ) {
-			m_pContextControl->CreateTab();
-			pState = CContextManager::Get()->GetCurrentContext();
-		}
-	}
-
-	CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromState(pState);
-	if (!isReconnect && controls) {
-		controls->site_bookmarks.reset();
-		m_pMenuBar->ClearBookmarks();
-	}
-
-	return pState->Connect(server, path);
-}
-
 void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 {
 	COptions::Get()->SetOption(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
@@ -2680,8 +2620,7 @@ void CMainFrame::OnOptionsChanged(changed_options_t const& options)
 		UpdateLayout();
 	}
 
-	bool const language_changed = options.test(OPTION_LANGUAGE);
-	if (options.test(OPTION_THEME) || options.test(OPTION_THEME_ICONSIZE) || language_changed) {
+	if (options.test(OPTION_THEME) || options.test(OPTION_THEME_ICONSIZE)) {
 		CreateMainToolBar();
 		if (m_pToolBar) {
 			m_pToolBar->UpdateToolbarState();
