@@ -2,25 +2,28 @@
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
-#include "RemoteListView.h"
-#include "commandqueue.h"
-#include "queue.h"
-#include "filezillaapp.h"
-#include "inputdialog.h"
 #include "chmoddialog.h"
-#include "filter.h"
-#include <algorithm>
-#include <wx/dcclient.h>
-#include <wx/dnd.h>
+#include "commandqueue.h"
 #include "dndobjects.h"
-#include "Options.h"
-#include "remote_recursive_operation.h"
-#include "edithandler.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
-#include <wx/clipbrd.h>
+#include "edithandler.h"
+#include "filezillaapp.h"
+#include "filter.h"
+#include "graphics.h"
+#include "inputdialog.h"
+#include "Options.h"
+#include "queue.h"
+#include "RemoteListView.h"
+#include "remote_recursive_operation.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
+
+#include <wx/clipbrd.h>
+#include <wx/dcclient.h>
+
+#include <algorithm>
+
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
@@ -290,28 +293,40 @@ protected:
 class CInfoText : public wxWindow
 {
 public:
-	CInfoText(wxWindow* parent, const wxString& text)
-		: wxWindow(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize),
-		m_text(text)
+	CInfoText(wxWindow* parent)
+		: m_tinter(*this)
 	{
+		Hide();
+		Create(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize);
+
 		SetForegroundColour(parent->GetForegroundColour());
 		SetBackgroundColour(parent->GetBackgroundColour());
 		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
+
+#ifdef __WXMSW__
+		if (GetLayoutDirection() != wxLayout_RightToLeft)
+			SetDoubleBuffered(true);
+#endif
 	}
 
-	void SetText(const wxString &text)
+	void SetText(wxString const& text)
 	{
-		if (text == m_text)
+		if (text == m_text) {
 			return;
+		}
 
 		m_text = text;
-
 		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
 	}
 
 	wxSize GetTextSize() const { return m_textSize; }
 
 	bool AcceptsFocus() const { return false; }
+
+	void SetBackgroundTint(wxColour const& colour) {
+		m_tinter.SetBackgroundTint(colour);
+	}
+
 protected:
 	wxString m_text;
 
@@ -327,6 +342,8 @@ protected:
 
 	wxSize m_textSize;
 
+	CWindowTinter m_tinter;
+
 	DECLARE_EVENT_TABLE()
 };
 
@@ -363,10 +380,12 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* p
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
 	state.RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
+	state.RegisterHandler(this, STATECHANGE_SERVER);
 
 	m_dropTarget = -1;
 
-	m_pInfoText = 0;
+	m_pInfoText = new CInfoText(this);
+
 	m_pDirectoryListing = 0;
 
 	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
@@ -391,6 +410,8 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* p
 	SetDropTarget(new CRemoteListViewDropTarget(this));
 
 	EnablePrefixSearch(true);
+
+	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
 }
 
 CRemoteListView::~CRemoteListView()
@@ -505,11 +526,6 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 
 	m_fileData.push_back(last);
 
-	std::list<bool> selected;
-	unsigned int start;
-	added_indexes.push_back(m_indexMapping.size());
-	start = added_indexes.front();
-
 	SetItemCount(m_indexMapping.size());
 	UpdateSelections_ItemsAdded(added_indexes);
 
@@ -1950,17 +1966,17 @@ void CRemoteListView::OnSize(wxSizeEvent& event)
 
 void CRemoteListView::RepositionInfoText()
 {
-	if (!m_pInfoText)
+	if (!m_pInfoText) {
 		return;
+	}
 
 	wxRect rect = GetClientRect();
-
 	wxSize size = m_pInfoText->GetTextSize();
 
-	if (m_indexMapping.empty())
+	if (!GetItemCount()) {
 		rect.y = 60;
-	else
-	{
+	}
+	else {
 		wxRect itemRect;
 		GetItemRect(0, itemRect);
 		rect.y = wxMax(60, itemRect.GetBottom() + 1);
@@ -1971,8 +1987,7 @@ void CRemoteListView::RepositionInfoText()
 
 	m_pInfoText->SetSize(rect);
 #ifdef __WXMSW__
-	if (GetLayoutDirection() != wxLayout_RightToLeft)
-	{
+	if (GetLayoutDirection() != wxLayout_RightToLeft) {
 		m_pInfoText->Refresh(true);
 		m_pInfoText->Update();
 	}
@@ -1990,6 +2005,10 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, co
 		wxASSERT(data2);
 		LinkIsNotDir(*(CServerPath*)data2, data);
 	}
+	else if (notification == STATECHANGE_SERVER) {
+		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
 		ApplyCurrentFilter();
@@ -2008,25 +2027,15 @@ void CRemoteListView::SetInfoText()
 			text = _("Empty directory listing");
 	}
 
+
 	if (text.empty()) {
-		delete m_pInfoText;
-		m_pInfoText = 0;
-		return;
+		m_pInfoText->Hide();
 	}
-
-	if (!m_pInfoText) {
-		m_pInfoText = new CInfoText(this, text);
-#ifdef __WXMSW__
-		if (GetLayoutDirection() != wxLayout_RightToLeft)
-			m_pInfoText->SetDoubleBuffered(true);
-#endif
-
+	else {
+		m_pInfoText->SetText(text);
 		RepositionInfoText();
-		return;
+		m_pInfoText->Show();
 	}
-
-	m_pInfoText->SetText(text);
-	RepositionInfoText();
 }
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
@@ -2537,7 +2546,7 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 	}
 	else {
 		for (auto const& entry : selected_item_list) {
-			urls = getUrl(server, path, entry.name);
+			urls += getUrl(server, path, entry.name);
 #ifdef __WXMSW__
 			urls += _T("\r\n");
 #else
