@@ -63,17 +63,21 @@ void CContextControl::CreateTab()
 
 		// See if we can reuse an existing context
 		for (size_t i = 0; i < m_context_controls.size(); i++) {
-			if (m_context_controls[i].tab_index != -1)
+			if (m_context_controls[i].used()) {
 				continue;
+			}
 
 			if (m_context_controls[i].pState->IsRemoteConnected() ||
 				!m_context_controls[i].pState->IsRemoteIdle())
+			{
 				continue;
+			}
 
 			pState = m_context_controls[i].pState;
 			m_context_controls.erase(m_context_controls.begin() + i);
-			if (m_current_context_controls > (int)i)
-				m_current_context_controls--;
+			if (m_current_context_controls > (int)i) {
+				--m_current_context_controls;
+			}
 			break;
 		}
 		if (!pState) {
@@ -84,10 +88,27 @@ void CContextControl::CreateTab()
 		}
 
 		// Restore last server and path
-		CServer last_server;
-		CServerPath last_path;
-		if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH)))
-			pState->SetLastServer(last_server, last_path);
+		auto * current = GetCurrentControls();
+		if (current) {
+			pState->SetLastSite(current->pState->GetLastSite(), current->pState->GetLastServerPath());
+		}
+		else {
+			CServer last_server;
+			CServerPath last_path;
+			if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH))) {
+				std::wstring last_site_path = COptions::Get()->GetOption(OPTION_LAST_CONNECTED_SITE).ToStdWstring();
+
+				std::unique_ptr<Site> site;
+				if (!last_site_path.empty()) {
+					site = CSiteManager::GetSiteByPath(last_site_path, false);
+				}
+				if (!site || site->m_server != last_server) {
+					site.reset(new Site);
+					site->m_server = last_server;
+				}
+				pState->SetLastSite(*site, last_path);
+			}				
+		}
 
 		CreateContextControls(*pState);
 
@@ -129,7 +150,7 @@ void CContextControl::CreateContextControls(CState& state)
 			m_tabs = new wxAuiNotebookEx();
 
 			wxSize splitter_size = m_context_controls[m_current_context_controls].pViewSplitter->GetSize();
-			m_tabs->Create(this, wxID_ANY, initial_position, splitter_size, wxNO_BORDER | wxAUI_NB_SCROLL_BUTTONS | wxAUI_NB_WINDOWLIST_BUTTON | wxAUI_NB_CLOSE_ON_ALL_TABS);
+			m_tabs->Create(this, wxID_ANY, initial_position, splitter_size, wxNO_BORDER | wxAUI_NB_SCROLL_BUTTONS | wxAUI_NB_WINDOWLIST_BUTTON | wxAUI_NB_CLOSE_ON_ALL_TABS | wxAUI_NB_TAB_MOVE);
 			m_tabs->SetExArtProvider();
 			m_tabs->SetSelectedFont(*wxNORMAL_FONT);
 			m_tabs->SetMeasuringFont(*wxNORMAL_FONT);
@@ -137,6 +158,7 @@ void CContextControl::CreateContextControls(CState& state)
 			m_context_controls[m_current_context_controls].pViewSplitter->Reparent(m_tabs);
 
 			m_tabs->AddPage(m_context_controls[m_current_context_controls].pViewSplitter, m_context_controls[m_current_context_controls].pState->GetTitle());
+			m_tabs->SetTabColour(0, m_context_controls[m_current_context_controls].pState->GetSite().m_colour);
 			ReplaceWindow(m_context_controls[m_current_context_controls].pViewSplitter, m_tabs);
 
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(CContextControl::OnTabChanged), 0, this);
@@ -158,7 +180,7 @@ void CContextControl::CreateContextControls(CState& state)
 		parent = m_tabs;
 	}
 
-	struct CContextControl::_context_controls context_controls;
+	CContextControl::_context_controls context_controls;
 
 	context_controls.pState = &state;
 	context_controls.pViewSplitter = new CSplitterWindowEx(parent, -1, initial_position, wxDefaultSize, wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
@@ -273,24 +295,9 @@ void CContextControl::CreateContextControls(CState& state)
 	state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
 
 	if (m_tabs) {
-		context_controls.tab_index = m_tabs->GetPageCount();
 		m_tabs->AddPage(context_controls.pViewSplitter, state.GetTitle());
-
-		// Copy reconnect and bookmark information
-		state.SetLastServer(
-			m_context_controls[m_current_context_controls].pState->GetLastServer(),
-			m_context_controls[m_current_context_controls].pState->GetLastServerPath());
-
-		context_controls.site_bookmarks = m_context_controls[m_current_context_controls].site_bookmarks;
 	}
 	else {
-		context_controls.tab_index = 0;
-		context_controls.site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
-
-		context_controls.site_bookmarks->path = COptions::Get()->GetOption(OPTION_LAST_CONNECTED_SITE);
-		CSiteManager::GetBookmarks(context_controls.site_bookmarks->path,
-			context_controls.site_bookmarks->bookmarks);
-
 		Initialize(context_controls.pViewSplitter);
 	}
 
@@ -299,22 +306,18 @@ void CContextControl::CreateContextControls(CState& state)
 
 void CContextControl::OnTabRefresh(wxCommandEvent&)
 {
-	if (m_right_clicked_tab == -1)
+	if (m_right_clicked_tab == -1) {
 		return;
+	}
 
-	for (size_t j = 0; j < m_context_controls.size(); j++)
-	{
-		if (m_context_controls[j].tab_index != m_right_clicked_tab)
-			continue;
-
-		m_context_controls[j].pState->RefreshLocal();
-		m_context_controls[j].pState->RefreshRemote();
-
-		break;
+	auto * controls = GetControlsFromTabIndex(m_right_clicked_tab);
+	if (controls) {
+		controls->pState->RefreshLocal();
+		controls->pState->RefreshRemote();
 	}
 }
 
-struct CContextControl::_context_controls* CContextControl::GetCurrentControls()
+ CContextControl::_context_controls* CContextControl::GetCurrentControls()
 {
 	if (m_current_context_controls == -1)
 		return 0;
@@ -322,11 +325,10 @@ struct CContextControl::_context_controls* CContextControl::GetCurrentControls()
 	return &m_context_controls[m_current_context_controls];
 }
 
-struct CContextControl::_context_controls* CContextControl::GetControlsFromState(CState* pState)
+CContextControl::_context_controls* CContextControl::GetControlsFromState(CState* pState)
 {
 	size_t i = 0;
-	for (i = 0; i < m_context_controls.size(); i++)
-	{
+	for (i = 0; i < m_context_controls.size(); ++i) {
 		if (m_context_controls[i].pState == pState)
 			return &m_context_controls[i];
 	}
@@ -335,24 +337,19 @@ struct CContextControl::_context_controls* CContextControl::GetControlsFromState
 
 bool CContextControl::CloseTab(int tab)
 {
-	if (!m_tabs)
+	if (!m_tabs) {
 		return false;
-	if (tab < 0)
-		return false;
-
-	size_t i = 0;
-	for (i = 0; i < m_context_controls.size(); i++)
-	{
-		if (m_context_controls[i].tab_index == tab)
-			break;
 	}
-	if (i == m_context_controls.size())
+	if (tab < 0 || static_cast<size_t>(tab) >= m_tabs->GetPageCount()) {
 		return false;
+	}
 
-	CState* pState = m_context_controls[i].pState;
 
-	if (!pState->m_pCommandQueue->Idle())
-	{
+	auto *const removeControls = GetControlsFromTabIndex(tab);
+
+	CState *const pState = removeControls->pState;
+
+	if (!pState->m_pCommandQueue->Idle()) {
 		if (wxMessageBoxEx(_("Cannot close tab while busy.\nCancel current operation and close tab?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
 			return false;
 	}
@@ -374,29 +371,22 @@ bool CContextControl::CloseTab(int tab)
 		m_tabs->Disconnect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(CContextControl::OnTabChanged), 0, this);
 
 		int keep = tab ? 0 : 1;
+
+		auto * keptControls = GetControlsFromTabIndex(keep);
 		m_tabs->RemovePage(keep);
 
-		size_t j;
-		for (j = 0; j < m_context_controls.size(); ++j) {
-			if (m_context_controls[j].tab_index != keep) {
-				continue;
-			}
+		CContextManager::Get()->SetCurrentContext(keptControls->pState);
 
-			break;
-		}
-
-		m_context_controls[j].pViewSplitter->Reparent(this);
-		ReplaceWindow(m_tabs, m_context_controls[j].pViewSplitter);
-		m_context_controls[j].pViewSplitter->Show();
-		m_context_controls[j].tab_index = 0;
+		keptControls->pViewSplitter->Reparent(this);
+		ReplaceWindow(m_tabs, keptControls->pViewSplitter);
+		keptControls->pViewSplitter->Show();
 
 		wxAuiNotebookEx *tabs = m_tabs;
 		m_tabs = 0;
 
-		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.reset();
+		removeControls->pViewSplitter = 0;
 
-		CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
+		CContextManager::Get()->SetCurrentContext(keptControls->pState);
 
 		tabs->Destroy();
 	}
@@ -407,21 +397,11 @@ bool CContextControl::CloseTab(int tab)
 				newsel = m_tabs->GetPageCount() - 2;
 			}
 
-			for (size_t j = 0; j < m_context_controls.size(); ++j) {
-				if (m_context_controls[j].tab_index != newsel) {
-					continue;
-				}
-				m_tabs->SetSelection(newsel);
-				CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
-			}
+			m_tabs->SetSelection(newsel);
+			CContextManager::Get()->SetCurrentContext(GetControlsFromTabIndex(newsel)->pState);
 		}
-		for (size_t j = 0; j < m_context_controls.size(); ++j) {
-			if (m_context_controls[j].tab_index > tab) {
-				--m_context_controls[j].tab_index;
-			}
-		}
-		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.reset();
+
+		removeControls->pViewSplitter = 0;
 		m_tabs->DeletePage(tab);
 	}
 
@@ -473,14 +453,13 @@ void CContextControl::OnTabContextCloseOthers(wxCommandEvent&)
 void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
 {
 	int tab = event.GetId();
-	if (tab < 0)
-	{
+	if (tab < 0) {
 		tab++;
 		int count = GetTabCount();
-		for (int i = count - 1; i >= 0; i--)
-		{
-			if (i != -tab)
+		for (int i = count - 1; i >= 0; --i) {
+			if (i != -tab) {
 				CloseTab(i);
+			}
 		}
 	}
 	else
@@ -491,17 +470,12 @@ void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
 void CContextControl::OnTabChanged(wxAuiNotebookEvent&)
 {
 	int i = m_tabs->GetSelection();
-	if (i < 0 || i >= (int)m_context_controls.size())
+	auto * const controls = GetControlsFromTabIndex(i);
+	if (!controls) {
 		return;
-
-	for (size_t j = 0; j < m_context_controls.size(); j++)
-	{
-		if (m_context_controls[j].tab_index != i)
-			continue;
-
-		CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
-		break;
 	}
+
+	CContextManager::Get()->SetCurrentContext(controls->pState);
 }
 
 void CContextControl::OnTabClosing(wxAuiNotebookEvent& event)
@@ -523,12 +497,26 @@ int CContextControl::GetTabCount() const
 	return m_tabs ? m_tabs->GetPageCount() : (m_context_controls.empty() ? 0 : 1);
 }
 
-struct CContextControl::_context_controls* CContextControl::GetControlsFromTabIndex(int i)
+CContextControl::_context_controls* CContextControl::GetControlsFromTabIndex(int i)
 {
-	for (size_t j = 0; j < m_context_controls.size(); j++)
-	{
-		if (m_context_controls[j].tab_index == i)
-			return &m_context_controls[j];
+	if (!m_tabs) {
+		if (i == 0 && !m_context_controls.empty()) {
+			for (auto & controls : m_context_controls) {
+				if (controls.pViewSplitter != 0) {
+					return &controls;
+				}
+			}
+		}
+		return 0;
+	}
+
+	wxWindow* page = m_tabs->GetPage(i);
+	if (page) {
+		for (auto & controls : m_context_controls) {
+			if (controls.pViewSplitter == page) {
+				return &controls;
+			}
+		}
 	}
 
 	return 0;
@@ -536,19 +524,21 @@ struct CContextControl::_context_controls* CContextControl::GetControlsFromTabIn
 
 bool CContextControl::SelectTab(int i)
 {
-	if (i < 0)
+	if (i < 0) {
 		return false;
+	}
 
-	if (!m_tabs)
-	{
-		if (i != 0)
+	if (!m_tabs) {
+		if (i != 0) {
 			return false;
+		}
 
 		return true;
 	}
 
-	if ((int)m_tabs->GetPageCount() <= i)
+	if ((int)m_tabs->GetPageCount() <= i) {
 		return false;
+	}
 
 	m_tabs->SetSelection(i);
 
@@ -557,8 +547,9 @@ bool CContextControl::SelectTab(int i)
 
 void CContextControl::AdvanceTab(bool forward)
 {
-	if (!m_tabs)
+	if (!m_tabs) {
 		return;
+	}
 
 	m_tabs->AdvanceTab(forward);
 }
@@ -580,12 +571,18 @@ void CContextControl::OnStateChange(CState* pState, t_statechange_notifications
 			m_current_context_controls = -1;
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		if (!m_tabs)
+		if (!m_tabs) {
 			return;
+		}
 
 		CContextControl::_context_controls* controls = GetControlsFromState(pState);
-		if (controls && controls->tab_index != -1)
-			m_tabs->SetPageText(controls->tab_index, controls->pState->GetTitle());
+		if (controls && controls->used()) {
+			int i = m_tabs->GetPageIndex(controls->pViewSplitter);
+			if (i != wxNOT_FOUND) {
+				m_tabs->SetTabColour(i, controls->pState->GetSite().m_colour);
+				m_tabs->SetPageText(i, controls->pState->GetTitle());
+			}
+		}
 	}
 }
 
