@@ -17,8 +17,8 @@ public:
 	struct _connected_site
 	{
 		CServer server;
-		wxString old_path;
-		wxString new_path;
+		std::wstring old_path;
+		std::wstring new_path;
 	};
 
 	/// Constructors
@@ -28,25 +28,25 @@ public:
 	// Creation. If pServer is set, it will cause a new item to be created.
 	bool Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, const CServer* pServer = 0);
 
-	bool GetServer(CSiteManagerItemData_Site& data);
-	wxString GetSitePath(bool stripBookmark = true);
-
+	bool GetServer(Site& data);
+	
 protected:
 	// Creates the controls and sizers
 	void CreateControls(wxWindow* parent);
 
 	bool Verify();
 	bool UpdateItem();
-	bool UpdateServer(CSiteManagerItemData_Site &server, const wxString& name);
-	bool UpdateBookmark(CSiteManagerItemData &bookmark, const CServer& server);
+	bool UpdateServer(Site &server, const wxString& name);
+	bool UpdateBookmark(Bookmark &bookmark, const CServer& server);
 	bool Load();
-	bool Save(TiXmlElement *pElement = 0, wxTreeItemId treeId = wxTreeItemId());
-	bool SaveChild(TiXmlElement *pElement, wxTreeItemId child);
+	bool Save(pugi::xml_node element = pugi::xml_node(), wxTreeItemId treeId = wxTreeItemId());
+	bool SaveChild(pugi::xml_node element, wxTreeItemId child);
 	void SetCtrlState();
 	bool LoadDefaultSites();
 
 	void SetProtocol(ServerProtocol protocol);
 	ServerProtocol GetProtocol() const;
+	LogonType GetLogonType() const;
 
 	bool IsPredefinedItem(wxTreeItemId item);
 
@@ -59,7 +59,7 @@ protected:
 
 	void RememberLastSelected();
 
-	wxString GetSitePath(wxTreeItemId item, bool stripBookmark = true);
+	std::wstring GetSitePath(wxTreeItemId item, bool stripBookmark = true);
 
 	void MarkConnectedSites();
 	void MarkConnectedSite(int connected_site);
@@ -88,6 +88,9 @@ protected:
 	void OnExportSelected(wxCommandEvent&);
 	void OnNewBookmark(wxCommandEvent&);
 	void OnBookmarkBrowse(wxCommandEvent&);
+	void OnKeyFileBrowse(wxCommandEvent&);
+
+	void SetControlVisibility(ServerProtocol protocol, LogonType type);
 
 	CInterProcessMutex* m_pSiteManagerMutex{};
 
