@@ -224,35 +224,6 @@ char *canonify(const char *name, int parent_only)
     }
 }
 
-/*
- * Return a pointer to the portion of str that comes after the last
- * slash (or backslash or colon, if `local' is TRUE).
- *
- * This function has the annoying strstr() property of taking a const
- * char * and returning a char *. You should treat it as if it was a
- * pair of overloaded functions, one mapping mutable->mutable and the
- * other const->const :-(
- */
-static char *stripslashes(const char *str, int local)
-{
-    char *p;
-
-    if (local) {
-        p = strchr(str, ':');
-        if (p) str = p+1;
-    }
-
-    p = strrchr(str, '/');
-    if (p) str = p+1;
-
-    if (local) {
-	p = strrchr(str, '\\');
-	if (p) str = p+1;
-    }
-
-    return (char *)str;
-}
-
 /*
  * qsort comparison routine for fxp_name structures. Sorts by real
  * file name.
@@ -486,7 +457,7 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
         pktin = sftp_wait_for_reply(req);
 	fxp_close_recv(pktin, req);
 
-	return 0;
+	return 2;
     }
 
     if (restart) {
@@ -522,7 +493,7 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
     xfer = xfer_download_init(fh, offset);
     while (!xfer_done(xfer)) {
 	void *vbuf;
-	int ret, len;
+	int len;
 	int wpos, wlen;
 
 	xfer_download_queue(xfer);
@@ -545,8 +516,10 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
 	    while (file && wpos < len) {
 		wlen = write_to_file(file, buf + wpos, len - wpos);
 		if (wlen <= 0) {
-		    fzprintf(shown_err ? sftpStatus : sftpError, "error while writing local file");
-		    shown_err = TRUE;
+		    if (!shown_err) {
+			fzprintf(sftpError, "error while writing local file");
+			shown_err = TRUE;
+		    }
 		    ret = 0;
 		    xfer_set_error(xfer);
 		    break;
@@ -718,7 +691,7 @@ int sftp_put_file(char *fname, char *outfname, int recurse, int restart)
     file = open_existing_file(fname, NULL, NULL, NULL, &permissions);
     if (!file) {
 	fzprintf(sftpError, "local: unable to open %s", fname);
-	return 0;
+	return 2;
     }
     attrs.flags = 0;
     PUT_PERMISSIONS(attrs, permissions);
@@ -2999,6 +2972,10 @@ int sftp_senddata(char *buf, int len)
     back->send(backhandle, buf, len);
     return 1;
 }
+int sftp_sendbuffer(void)
+{
+    return back->sendbuffer(backhandle);
+}
 
 /*
  *  Short description of parameters.
@@ -3024,10 +3001,13 @@ static void usage(void)
     printf("  -C        enable compression\n");
     printf("  -i key    private key file for user authentication\n");
     printf("  -noagent  disable use of Pageant\n");
+    printf("  -agent    enable use of Pageant\n");
     printf("  -hostkey aa:bb:cc:...\n");
     printf("            manually specify a host key (may be repeated)\n");
-    printf("  -agent    enable use of Pageant\n");
     printf("  -batch    disable all interactive prompts\n");
+    printf("  -sshlog file\n");
+    printf("  -sshrawlog file\n");
+    printf("            log protocol details to a file\n");
     cleanup_exit(1);
 }
 
@@ -3095,6 +3075,7 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
         conf_set_int(conf, CONF_port, 22);
     }
 
+#if 0
     /*
      * If saved session / Default Settings says SSH-1 (`1 only' or `1'),
      * then change it to SSH-2, on the grounds that that's more likely to
@@ -3103,6 +3084,10 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
      */
     if ((conf_get_int(conf, CONF_sshprot) & ~1) != 2)   /* is it 2 or 3? */
 	conf_set_int(conf, CONF_sshprot, 2);
+#else
+    /* FZ: Require SSH2 */
+    conf_set_int(conf, CONF_sshprot, 3);
+#endif
 
     /*
      * Enact command-line overrides.
@@ -3378,6 +3363,34 @@ int psftp_main(int argc, char *argv[])
     // FZ: Set proxy to none
     conf_set_int(conf, CONF_proxy_type, PROXY_NONE);
 
+    // FZ: Re-order ciphers so that old and insecure algorithms are always below the warning level
+    {
+	// Find position of warning level
+	int warn = -1;
+	for (i = 0; i < CIPHER_MAX && warn == -1; ++i) {
+	    int cipher = conf_get_int_int(conf, CONF_ssh_cipherlist, i);
+	    if (cipher == CIPHER_WARN) {
+		warn = i;
+	    }
+	}
+
+	if (warn != -1 && warn < CIPHER_MAX) {
+	    for (i = warn - 1; i >= 0; --i) {
+		int const cipher = conf_get_int_int(conf, CONF_ssh_cipherlist, i);
+		if (cipher == CIPHER_ARCFOUR || cipher == CIPHER_DES) {
+		    int j;
+		    // Bubble it down
+		    for (j = i; j < warn; ++j) {
+			int swap = conf_get_int_int(conf, CONF_ssh_cipherlist, j + 1);
+			conf_set_int_int(conf, CONF_ssh_cipherlist, j, swap);
+		    }
+		    conf_set_int_int(conf, CONF_ssh_cipherlist, warn, cipher);
+		    --warn;
+		}
+	    }
+	}
+    }
+
     for (i = 1; i < argc; i++) {
 	int ret;
 	if (argv[i][0] != '-') {
@@ -3426,6 +3439,8 @@ int psftp_main(int argc, char *argv[])
     argv += i;
     back = NULL;
 
+    platform_psftp_post_option_setup();
+
     /*
      * If the loaded session provides a hostname, and a hostname has not
      * otherwise been specified, pop it in `userhost' so that
