@@ -152,33 +152,45 @@ void update_specials_menu(void *frontend)
  */
 int askalg(void *frontend, const char *algtype, const char *algname,
 	   void (*callback)(void *ctx, int result), void *ctx)
+{
+    fzprintf(sftpError, "The first %s supported by the server is %s, which is no longer secure. Aborting connection.", algtype, algname);
+    return 0;
+}
+
+int askhk(void *frontend, const char *algname, const char *betteralgs,
+          void (*callback)(void *ctx, int result), void *ctx)
 {
     HANDLE hin;
     DWORD savemode, i;
 
-    static const char msg[] =
-	"The first %s supported by the server is\n"
-	"%s, which is below the configured warning threshold.\n"
-	"Continue with connection? (y/n) ";
+    /*static const char msg[] =
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
+	"Continue with connection? (y/n) ";*/
     static const char msg_batch[] =
-	"The first %s supported by the server is\n"
-	"%s, which is below the configured warning threshold.\n"
+	"The first host key type we have stored for this server\n"
+	"is %s, which is below the configured warning threshold.\n"
+	"The server also provides the following types of host key\n"
+        "above the threshold, which we do not have stored:\n"
+        "%s\n"
 	"Connection abandoned.\n";
-    static const char abandoned[] = "Connection abandoned.\n";
+    static const char abandoned[] = "Connection abandoned.";
 
     char line[32];
 
     if (console_batch_mode) {
-	fprintf(stderr, msg_batch, algtype, algname);
+	fprintf(stderr, msg_batch, algname, betteralgs);
 	return 0;
     }
 
-    fprintf(stderr, msg, algtype, algname);
-    fflush(stderr);
+    fzprintf_raw(sftpRequest, "%d%s\n%s\n", (int)sftpReqHostkeyBetteralg, algname, betteralgs);
 
     hin = GetStdHandle(STD_INPUT_HANDLE);
     GetConsoleMode(hin, &savemode);
-    SetConsoleMode(hin, (savemode | ENABLE_ECHO_INPUT |
+    SetConsoleMode(hin, (savemode |
 			 ENABLE_PROCESSED_INPUT | ENABLE_LINE_INPUT));
     ReadFile(hin, line, sizeof(line) - 1, &i, NULL);
     SetConsoleMode(hin, savemode);
@@ -186,7 +198,7 @@ int askalg(void *frontend, const char *algtype, const char *algname,
     if (line[0] == 'y' || line[0] == 'Y') {
 	return 1;
     } else {
-	fprintf(stderr, abandoned);
+	fzprintf(sftpError, abandoned);
 	return 0;
     }
 }
@@ -275,9 +287,11 @@ void pgp_fingerprints(void)
 	  "one. See the manual for more information.\n"
 	  "(Note: these fingerprints have nothing to do with SSH!)\n"
 	  "\n"
-	  "PuTTY Master Key (RSA), 1024-bit:\n"
+	  "PuTTY Master Key as of 2015 (RSA, 4096-bit):\n"
+	  "  " PGP_MASTER_KEY_FP "\n\n"
+	  "Original PuTTY Master Key (RSA, 1024-bit):\n"
 	  "  " PGP_RSA_MASTER_KEY_FP "\n"
-	  "PuTTY Master Key (DSA), 1024-bit:\n"
+	  "Original PuTTY Master Key (DSA, 1024-bit):\n"
 	  "  " PGP_DSA_MASTER_KEY_FP "\n", stdout);
 }
 
@@ -332,7 +346,7 @@ int console_get_userpass_input(prompts_t *p,
     /*
      * And if we have anything to print, we need standard output.
      */
-    if ((p->name_reqd && p->name) || p->instruction || p->n_prompts) {
+    if ((p->name) || p->instruction || p->n_prompts) {
 	hout = GetStdHandle(STD_OUTPUT_HANDLE);
 	if (hout == INVALID_HANDLE_VALUE) {
 	    fprintf(stderr, "Cannot get standard output handle\n");
@@ -344,7 +358,7 @@ int console_get_userpass_input(prompts_t *p,
      * Preamble.
      */
     /* We only print the `name' caption if we have to... */
-    if (p->name_reqd && p->name)
+    if (p->name)
 	fzprintf_raw_untrusted(sftpRequestPreamble, p->name);
     else
 	fzprintf_raw_untrusted(sftpRequestPreamble, "");
